@@ -64,7 +64,7 @@ def column_clause(dictionary, constraints=None, all_char_as_varchar=True):
     
     clause = '%s %s' % (name, data_type)
     if constraints is not None:
-        allowed_constraints = ['NULL', 'NOT NULL', 'PRIMARY KEY', 'UNIQUE']
+        allowed_constraints = ['NULL', 'NOT NULL', 'PRIMARY KEY', 'UNIQUE', 'DEFAULT NULL']
         constrained_by = [x for x in constraints if x.upper() in allowed_constraints]
         if len(constrained_by) > 0:
             clause = '%s %s' % (clause, ' '.join(constrained_by))
@@ -206,7 +206,7 @@ def create_table(cnxn, table_name, meta, schema=None, pk_fields=None,
     return True
 
 
-def long_table_sql(table_name, category, schema=None, ine=True):
+def long_table_sql(table_name, category, schema=None, ine=True, constraints=['DEFAULT NULL']):
     """Wraps table_sql(). Generates CREATE TABLE statement for a long table of the specified category.
     
     Also creates indexes on KEY, VISITLINK, YEAR, and STATE.
@@ -543,21 +543,116 @@ def pg_castcoltype(archtype, length, scale=None, all_char_as_varchar=True):
     return data_type
 
 
-def pg_shovel(cnxn, tbl_source, tbl_destination, field_lst, preserve_source=True):
+def pg_wtl_shovel(cnxn, meta_df, category, tbl_source,
+        tbl_destination, extra_fields=['state', 'year', 'key']):
+    """Shovels wide things into long things
+    
+    category is 'CHGS' only, for now
+    """
+    lm = parser.LONG_MAPS[category]
+
+    reverse_lm = {}
+    field_s = meta_df.field
+
+    for des_col, in_col_lst in lm.iteritems():
+        matched = False
+        for in_candidate in in_col_lst:
+            # Look at the candidates for incoming field
+            # and see if there is a match for the candidate
+            # among the incoming meta columns, minus one char.
+            # This is to accommodate things like CHG1, so that
+            # it matches CHG.
+            if (
+                not matched and
+                in_candidate in field_s.map(lambda x: str(x)[:-1]).values
+                ):
+                reverse_lm[in_candidate] = des_col
+                matched = True
+
+    assert all([isinstance(v, str) for k, v in reverse_lm.iteritems()]), \
+        "Somehow the match in the reverse long map is not a string."
+
+    # We tolerate missing cols (e.g., no RATE provided), but not extra cols.
+    assert len(reverse_lm) <= len(lm), \
+        "Ended up with more reverse long map entries than long map entries."
+
+
+    # Get the max number of each variety of column
+    maxes = []
+    for k in reverse_lm.keys():
+        fields_containing = field_s[ field_s.map(lambda x: k in x) ]
+        nums = fields_containing.map(lambda x: x.split(k)[1])
+        maxes.append(nums.astype(int).max())
+
+    # Make sure we have the same number of each variety of column
+    assert all([x == maxes[0] for x in maxes]), \
+        "One or more columns to be longed has an unusual number of columns. Please inspect the data."
+    groups = maxes[0]
+
+    # Select groups of these columns, up to n, to use for input
+    shoveled = 0
+
+    for x in xrange(1, groups+1):
+        cols_select = ['%s%s' % (k, x) for k in reverse_lm.keys()]
+        cols_as = reverse_lm.values()
+        
+        if isinstance(extra_fields, list) and len(extra_fields) > 0:
+            # some extra fields are requested; likely state, year, and key
+            cols_select += extra_fields
+            cols_as += extra_fields
+        
+        # only drop tables after doing the final shovel
+        if x == groups:
+            preserve_source = True
+        else:
+            preserve_source = False
+        
+        if category == 'CHGS':
+            wnn = ['CHG%s' % x]
+        else:
+            wnn = None
+        
+        shoveled += pg_shovel(cnxn, tbl_source, tbl_destination, cols_select,
+            cols_as, where_not_null=wnn, preserve_source=preserve_source)
+    
+    return shoveled
+
+
+def pg_shovel(cnxn, tbl_source, tbl_destination, fields_in, fields_out=False,
+              where_not_null=None, preserve_source=False):
     """Attempts to move everything in tbl_source to tbl_destination using cnxn.
     
     The SQL used here is supported for sure in PostgreSQL but untested in other engines.
 
+    fields_out must either be False or a list with length len(fields_in). These will be used as "SELECT {field_in} AS {field_out}" to map values to differently named columns.
+    
     If preserve_source is False, the table will be dropped after the transfer.
     """
     cursor = cnxn.cursor()
     
-    fields = ', '.join(field_lst)
+    if fields_out:
+        assert len(fields_out) == len(fields_in), \
+            "If provided, fields_out must have the same length as fields_in."
+        select_fields = ', '.join(['%s AS %s' % cols for cols in zip(fields_in, fields_out)])
+        insert_fields = ', '.join(fields_out)
+    else:
+        select_fields = ', '.join(fields_in)
+        insert_fields = ', '.join(fields_in)
     
     stmt = '''
         INSERT INTO %s (%s)
-            SELECT %s FROM %s;
-        ''' % (str(tbl_destination), fields, fields, str(tbl_source))
+            SELECT %s FROM %s
+        ''' % (str(tbl_destination), insert_fields, select_fields, str(tbl_source))
+    
+    if where_not_null is not None:
+        assert isinstance(where_not_null, list), \
+            "If provided, where_not_null must be a list (got %s)." % (type(where_not_null))
+        
+        assert all([x in fields_in for x in where_not_null]), \
+            "If provided, all items in where_not_null must be items in fields_in (got %s and %s, respectively)." % (where_not_null, field_in)
+        
+        stmt += 'WHERE (%s)' % (' AND '.join(['%s IS NOT NULL' % wnn for wnn in where_not_null]))
+    
     
     cursor.execute(stmt)
     cnxn.commit()
